package genius.core.uncertainty;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import genius.core.bidding.BidDetails;
import genius.core.boaframework.SortedOutcomeSpace;

public class ComparisonGenerator {

	private SortedOutcomeSpace outcomeSpace;
	private List<OutcomeComparison> allComparisons;
	
	public ComparisonGenerator(SortedOutcomeSpace outcomeSpace){
		this.outcomeSpace = outcomeSpace;
		this.allComparisons = generateAllOutcomeComparisons();
	}
	
	public List<OutcomeComparison> generateAllOutcomeComparisons() {
		List<BidDetails> allBids = this.outcomeSpace.getAllOutcomes();
		List<OutcomeComparison> allComparisons = new ArrayList<OutcomeComparison>();
			for (int i = 0; i < allBids.size(); i++) {
				for (int j = i+1; j < allBids.size(); j++) {
					allComparisons.add(new OutcomeComparison(allBids.get(i), allBids.get(j)));
				}
			}
			Collections.shuffle(allComparisons);
		return allComparisons;
	}
	
	public List<OutcomeComparison> generateComparisons(int amount) {
		List<OutcomeComparison> comparisons = new ArrayList<OutcomeComparison>();
		for (int i = 0; i < amount; i++) {
			comparisons.add(allComparisons.get(i));
		}
		return comparisons;
	}
	public List<OutcomeComparison> generateComparisonsWithError(int amount, double error) {
		List<OutcomeComparison> comparisons = generateComparisons(amount);
		int wrongComparisonsAmount = (int) (error * comparisons.size());
		for (int i = 0; i < wrongComparisonsAmount; i++) {
			if (comparisons.get(i).getComparisonResult() == 1)  comparisons.get(i).setComparisonResult(-1);
			else  comparisons.get(i).setComparisonResult(+1);
		}
		System.out.println(wrongComparisonsAmount);
		return comparisons;
	}
	
	public List<OutcomeComparison> getAllComparisons() {
		return allComparisons;
	}
}
